@@ -19,8 +19,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -29,6 +29,7 @@
 ;;; Code:
 
 (require 'tramp)
+(require 'tramp-cache)
 
 ;; Pacify byte-compiler
 (eval-when-compile
@@ -38,15 +39,8 @@
 ;; Avoid byte-compiler warnings if the byte-compiler supports this.
 ;; Currently, XEmacs supports this.
 (eval-when-compile
-  (when (fboundp 'byte-compiler-options)
-    (let (unused-vars) ; Pacify Emacs byte-compiler
-      (defalias 'warnings 'identity) ; Pacify Emacs byte-compiler
-      (byte-compiler-options (warnings (- unused-vars))))))
-
-;; XEmacs byte-compiler raises warning abouts `last-coding-system-used'.
-(eval-when-compile
-  (unless (boundp 'last-coding-system-used)
-    (defvar last-coding-system-used nil)))
+  (when (featurep 'xemacs)
+      (byte-compiler-options (warnings (- unused-vars)))))
 
 ;; Define SMB method ...
 (defcustom tramp-smb-method "smb"
@@ -77,7 +71,7 @@
   :group 'tramp
   :type 'string)
 
-(defconst tramp-smb-prompt "^smb: \\S-+> \\|^\\s-+Server\\s-+Comment$"
+(defconst tramp-smb-prompt "^smb: .+> \\|^\\s-+Server\\s-+Comment$"
   "Regexp used as prompt in smbclient.")
 
 (defconst tramp-smb-errors
@@ -212,8 +206,12 @@ pass to the OPERATION."
 
 (defun tramp-smb-not-handled (operation &rest args)
   "Default handler for all functions which are disrecarded."
-  (tramp-message 10 "Won't be handled: %s %s" operation args)
-  nil)
+  (when (stringp (car args))
+    (with-parsed-tramp-file-name (car args) nil
+      (tramp-message-for-buffer
+       tramp-smb-method user host
+       2 "Won't be handled: %s %s" operation args)
+      nil)))
 
 (defun tramp-smb-handle-copy-file
   (filename newname &optional ok-if-already-exists keep-date)
@@ -237,27 +235,28 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	(error "copy-file: file %s already exists" newname))
 
       (with-parsed-tramp-file-name newname nil
-	(save-excursion
-	  (let ((share (tramp-smb-get-share localname))
-		(file (tramp-smb-get-localname localname t)))
-	    (unless share
-	      (error "Target `%s' must contain a share name" filename))
-	    (tramp-smb-maybe-open-connection user host share)
-	    (tramp-message-for-buffer tramp-smb-method user host
-	     5 "Copying file %s to file %s..." filename newname)
-	    (if (tramp-smb-send-command
-		 user host (format "put %s \"%s\"" filename file))
-		(tramp-message-for-buffer tramp-smb-method user host
-		 5 "Copying file %s to file %s...done" filename newname)
-	      (error "Cannot copy `%s'" filename))))))))
+	(tramp-cache-flush-file tramp-smb-method user host localname)
+	(let ((share (tramp-smb-get-share localname))
+	      (file (tramp-smb-get-localname localname t)))
+	  (unless share
+	    (error "Target `%s' must contain a share name" filename))
+	  (tramp-smb-maybe-open-connection user host share)
+	  (tramp-message-for-buffer
+	   tramp-smb-method user host
+	   4 "Copying file %s to file %s..." filename newname)
+	  (if (tramp-smb-send-command
+	       user host (format "put %s \"%s\"" filename file))
+	      (tramp-message-for-buffer
+	       tramp-smb-method user host
+	       4 "Copying file %s to file %s...done" filename newname)
+	    (error "Cannot copy `%s'" filename)))))))
 
 (defun tramp-smb-handle-delete-directory (directory)
   "Like `delete-directory' for tramp files."
   (setq directory (directory-file-name (expand-file-name directory)))
-  (unless (file-exists-p directory)
-    (error "Cannot delete non-existing directory `%s'" directory))
-  (with-parsed-tramp-file-name directory nil
-    (save-excursion
+  (when (file-exists-p directory)
+    (with-parsed-tramp-file-name directory nil
+      (tramp-cache-flush-directory tramp-smb-method user host localname)
       (let ((share (tramp-smb-get-share localname))
 	    (dir (tramp-smb-get-localname (file-name-directory localname) t))
 	    (file (file-name-nondirectory localname)))
@@ -274,15 +273,12 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 (defun tramp-smb-handle-delete-file (filename)
   "Like `delete-file' for tramp files."
   (setq filename (expand-file-name filename))
-  (unless (file-exists-p filename)
-    (error "Cannot delete non-existing file `%s'" filename))
-  (with-parsed-tramp-file-name filename nil
-    (save-excursion
+  (when (file-exists-p filename)
+    (with-parsed-tramp-file-name filename nil
+      (tramp-cache-flush-file tramp-smb-method user host localname)
       (let ((share (tramp-smb-get-share localname))
 	    (dir (tramp-smb-get-localname (file-name-directory localname) t))
 	    (file (file-name-nondirectory localname)))
-	(unless (file-exists-p filename)
-	  (error "Cannot delete non-existing file `%s'" filename))
 	(tramp-smb-maybe-open-connection user host share)
 	(if (and
 	     (tramp-smb-send-command user host (format "cd \"%s\"" dir))
@@ -296,30 +292,22 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 (defun tramp-smb-handle-directory-files
   (directory &optional full match nosort)
   "Like `directory-files' for tramp files."
-  (setq directory (directory-file-name (expand-file-name directory)))
-  (with-parsed-tramp-file-name directory nil
-    (save-excursion
-      (let* ((share (tramp-smb-get-share localname))
-	     (file (tramp-smb-get-localname localname nil))
-	     (entries (tramp-smb-get-file-entries user host share file)))
-	;; Just the file names are needed
-	(setq entries (mapcar 'car entries))
-	;; Discriminate with regexp
-	(when match
-	  (setq entries
-		(delete nil
-			(mapcar (lambda (x) (when (string-match match x) x))
-				entries))))
-	;; Make absolute localnames if necessary
-	(when full
-	  (setq entries
-		(mapcar (lambda (x)
-			  (concat (file-name-as-directory directory) x))
-			entries)))
-	;; Sort them if necessary
-	(unless nosort (setq entries (sort entries 'string-lessp)))
-	;; That's it
-	entries))))
+  (let ((result
+	 (mapcar
+	  (lambda (x)
+	    (directory-file-name
+	     (if full (concat (file-name-as-directory directory) x) x)))
+	  (file-name-all-completions "" directory))))
+    ;; Discriminate with regexp
+    (when match
+      (setq result
+	    (delete nil
+		    (mapcar (lambda (x) (when (string-match match x) x))
+			    result))))
+    ;; Sort them if necessary
+    (unless nosort (setq result (sort result 'string-lessp)))
+    ;; That's it
+    result))
 
 (defun tramp-smb-handle-directory-files-and-attributes
   (directory &optional full match nosort id-format)
@@ -327,15 +315,17 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
   (mapcar
    (lambda (x)
      ;; We cannot call `file-attributes' for backward compatibility reasons.
-     ;; Its optional parameter ID-FORMAT is introduced with Emacs 21.4.
+     ;; Its optional parameter ID-FORMAT is introduced with Emacs 22.
      (cons x (tramp-smb-handle-file-attributes
 	(if full x (concat (file-name-as-directory directory) x)) id-format)))
    (directory-files directory full match nosort)))
- 
+
 (defun tramp-smb-handle-file-attributes (filename &optional id-format)
   "Like `file-attributes' for tramp files."
   (with-parsed-tramp-file-name filename nil
-    (save-excursion
+    (with-cache-data
+	tramp-smb-method user host localname
+	(format "file-attributes-%s" id-format)
       (let* ((share (tramp-smb-get-share localname))
 	     (file (tramp-smb-get-localname localname nil))
 	     (entries (tramp-smb-get-file-entries user host share file))
@@ -346,26 +336,26 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	     (inode (tramp-smb-get-inode share file))
 	     (device (tramp-get-device tramp-smb-method user host)))
 
-	; check result
+        ; check result
 	(when entry
 	  (list (and (string-match "d" (nth 1 entry))
-		     t)         ;0 file type
-		-1		;1 link count
-		uid		;2 uid
-		gid		;3 gid
-		'(0 0)		;4 atime
-		(nth 3 entry)	;5 mtime
-		'(0 0)		;6 ctime
-		(nth 2 entry)   ;7 size
-		(nth 1 entry)   ;8 mode
-		nil		;9 gid weird
-		inode		;10 inode number
-		device))))))	;11 file system number
+		     t)       ;0 file type
+		-1	      ;1 link count
+		uid	      ;2 uid
+		gid	      ;3 gid
+		'(0 0)	      ;4 atime
+		(nth 3 entry) ;5 mtime
+		'(0 0)	      ;6 ctime
+		(nth 2 entry) ;7 size
+		(nth 1 entry) ;8 mode
+		nil	      ;9 gid weird
+		inode	      ;10 inode number
+		device))))))  ;11 file system number
 
 (defun tramp-smb-handle-file-directory-p (filename)
   "Like `file-directory-p' for tramp files."
   (with-parsed-tramp-file-name filename nil
-    (save-excursion
+    (with-cache-data tramp-smb-method user host localname "file-directory-p"
       (let* ((share (tramp-smb-get-share localname))
 	     (file (tramp-smb-get-localname localname nil))
 	     (entries (tramp-smb-get-file-entries user host share file))
@@ -378,7 +368,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 (defun tramp-smb-handle-file-exists-p (filename)
   "Like `file-exists-p' for tramp files."
   (with-parsed-tramp-file-name filename nil
-    (save-excursion
+    (with-cache-data tramp-smb-method user host localname "file-exists-p"
       (let* ((share (tramp-smb-get-share localname))
 	     (file (tramp-smb-get-localname localname nil))
 	     (entries (tramp-smb-get-file-entries user host share file)))
@@ -389,35 +379,37 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 (defun tramp-smb-handle-file-local-copy (filename)
   "Like `file-local-copy' for tramp files."
   (with-parsed-tramp-file-name filename nil
-    (save-excursion
-      (let ((share (tramp-smb-get-share localname))
-	    (file (tramp-smb-get-localname localname t))
-	    (tmpfil (tramp-make-temp-file)))
-	(unless (file-exists-p filename)
-	  (error "Cannot make local copy of non-existing file `%s'" filename))
-	(tramp-message-for-buffer tramp-smb-method user host
-	 5 "Fetching %s to tmp file %s..." filename tmpfil)
-	(tramp-smb-maybe-open-connection user host share)
-	(if (tramp-smb-send-command
-	     user host (format "get \"%s\" %s" file tmpfil))
-	    (tramp-message-for-buffer tramp-smb-method user host
-	     5 "Fetching %s to tmp file %s...done" filename tmpfil)
-	  (error "Cannot make local copy of file `%s'" filename))
-	tmpfil))))
+    (let ((share (tramp-smb-get-share localname))
+	  (file (tramp-smb-get-localname localname t))
+	  (tmpfil (tramp-make-temp-file)))
+      (unless (file-exists-p filename)
+	(error "Cannot make local copy of non-existing file `%s'" filename))
+      (tramp-message-for-buffer
+       tramp-smb-method user host
+       4 "Fetching %s to tmp file %s..." filename tmpfil)
+      (tramp-smb-maybe-open-connection user host share)
+      (if (tramp-smb-send-command
+	   user host (format "get \"%s\" %s" file tmpfil))
+	  (tramp-message-for-buffer
+	   tramp-smb-method user host
+	   4 "Fetching %s to tmp file %s...done" filename tmpfil)
+	(error "Cannot make local copy of file `%s'" filename))
+      tmpfil)))
 
 ;; This function should return "foo/" for directories and "bar" for
 ;; files.
 (defun tramp-smb-handle-file-name-all-completions (filename directory)
   "Like `file-name-all-completions' for tramp files."
-  (with-parsed-tramp-file-name directory nil
-    (save-match-data
-      (save-excursion
-	(let* ((share (tramp-smb-get-share localname))
-	       (file (tramp-smb-get-localname localname nil))
-	       (entries (tramp-smb-get-file-entries user host share file)))
-
-	  (all-completions
-	   filename
+  (all-completions
+   filename
+   (with-parsed-tramp-file-name directory nil
+     (with-cache-data tramp-smb-method user host localname
+		      "file-name-all-completions"
+       (save-match-data
+	 (let* ((share (tramp-smb-get-share localname))
+		(file (file-name-as-directory
+		       (tramp-smb-get-localname localname nil)))
+		(entries (tramp-smb-get-file-entries user host share file)))
 	   (mapcar
 	    (lambda (x)
 	      (list
@@ -441,7 +433,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	(and (file-exists-p dir)
 	     (file-writable-p dir)))
     (with-parsed-tramp-file-name filename nil
-      (save-excursion
+      (with-cache-data tramp-smb-method user host localname "file-writable-p"
 	(let* ((share (tramp-smb-get-share localname))
 	       (file (tramp-smb-get-localname localname nil))
 	       (entries (tramp-smb-get-file-entries user host share file))
@@ -461,13 +453,14 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
     ;; this function is called with a non-directory ...
     (setq filename (file-name-as-directory filename)))
   (with-parsed-tramp-file-name filename nil
+    (tramp-cache-flush-file tramp-smb-method user host localname)
     (save-match-data
       (let* ((share (tramp-smb-get-share localname))
 	     (file (tramp-smb-get-localname localname nil))
 	     (entries (tramp-smb-get-file-entries user host share file)))
 
 	;; Delete dummy "" entry, useless entries
-	(setq entries 
+	(setq entries
 	      (if (file-directory-p filename)
 		  (delq (assoc "" entries) entries)
 		;; We just need the only and only entry FILENAME.
@@ -559,17 +552,19 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 	  (error "rename-file: file %s already exists" newname))
 
       (with-parsed-tramp-file-name newname nil
-	(save-excursion
-	  (let ((share (tramp-smb-get-share localname))
-		(file (tramp-smb-get-localname localname t)))
-	    (tramp-smb-maybe-open-connection user host share)
-	    (tramp-message-for-buffer tramp-smb-method user host
-	     5 "Copying file %s to file %s..." filename newname)
-	    (if (tramp-smb-send-command
-		 user host (format "put %s \"%s\"" filename file))
-		(tramp-message-for-buffer tramp-smb-method user host
-		 5 "Copying file %s to file %s...done" filename newname)
-	      (error "Cannot rename `%s'" filename)))))))
+	(tramp-cache-flush-file tramp-smb-method user host localname)
+	(let ((share (tramp-smb-get-share localname))
+	      (file (tramp-smb-get-localname localname t)))
+	  (tramp-smb-maybe-open-connection user host share)
+	  (tramp-message-for-buffer
+	   tramp-smb-method user host
+	   4 "Copying file %s to file %s..." filename newname)
+	  (if (tramp-smb-send-command
+	       user host (format "put %s \"%s\"" filename file))
+	      (tramp-message-for-buffer
+	       tramp-smb-method user host
+	       4 "Copying file %s to file %s...done" filename newname)
+	    (error "Cannot rename `%s'" filename))))))
 
   (delete-file filename))
 
@@ -593,50 +588,39 @@ Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
                               filename))
       (error "File not overwritten")))
   (with-parsed-tramp-file-name filename nil
-    (save-excursion
-      (let ((share (tramp-smb-get-share localname))
-	    (file (tramp-smb-get-localname localname t))
-	    (curbuf (current-buffer))
-	    ;; We use this to save the value of `last-coding-system-used'
-	    ;; after writing the tmp file.  At the end of the function,
-	    ;; we set `last-coding-system-used' to this saved value.
-	    ;; This way, any intermediary coding systems used while
-	    ;; talking to the remote shell or suchlike won't hose this
-	    ;; variable.  This approach was snarfed from ange-ftp.el.
-	    coding-system-used
-	    tmpfil)
-	;; Write region into a tmp file.
-	(setq tmpfil (tramp-make-temp-file))
-	;; We say `no-message' here because we don't want the visited file
-	;; modtime data to be clobbered from the temp file.  We call
-	;; `set-visited-file-modtime' ourselves later on.
-	(tramp-run-real-handler
-	 'write-region
-	 (if confirm ; don't pass this arg unless defined for backward compat.
-	     (list start end tmpfil append 'no-message lockname confirm)
-	   (list start end tmpfil append 'no-message lockname)))
-	;; Now, `last-coding-system-used' has the right value.  Remember it.
-	(when (boundp 'last-coding-system-used)
-	  (setq coding-system-used last-coding-system-used))
+    (tramp-cache-flush-file tramp-smb-method user host localname)
+    (let ((share (tramp-smb-get-share localname))
+	  (file (tramp-smb-get-localname localname t))
+	  (curbuf (current-buffer))
+	  tmpfil)
+      ;; Write region into a tmp file.
+      (setq tmpfil (tramp-make-temp-file))
+      ;; We say `no-message' here because we don't want the visited file
+      ;; modtime data to be clobbered from the temp file.  We call
+      ;; `set-visited-file-modtime' ourselves later on.
+      (tramp-run-real-handler
+       'write-region
+       (if confirm ; don't pass this arg unless defined for backward compat.
+	   (list start end tmpfil append 'no-message lockname confirm)
+	 (list start end tmpfil append 'no-message lockname)))
 
-	(tramp-smb-maybe-open-connection user host share)
-	(tramp-message-for-buffer tramp-smb-method user host
-	 5 "Writing tmp file %s to file %s..." tmpfil filename)
-	(if (tramp-smb-send-command
-	     user host (format "put %s \"%s\"" tmpfil file))
-	    (tramp-message-for-buffer tramp-smb-method user host
-	     5 "Writing tmp file %s to file %s...done" tmpfil filename)
-	  (error "Cannot write `%s'" filename))
+      (tramp-smb-maybe-open-connection user host share)
+      (tramp-message-for-buffer
+       tramp-smb-method user host
+       5 "Writing tmp file %s to file %s..." tmpfil filename)
+      (if (tramp-smb-send-command
+	   user host (format "put %s \"%s\"" tmpfil file))
+	  (tramp-message-for-buffer
+	   tramp-smb-method user host
+	   5 "Writing tmp file %s to file %s...done" tmpfil filename)
+	(error "Cannot write `%s'" filename))
 
-	(delete-file tmpfil)
-	(unless (equal curbuf (current-buffer))
-	  (error "Buffer has changed from `%s' to `%s'"
-		 curbuf (current-buffer)))
-	(when (eq visit t)
-	  (set-visited-file-modtime))
-	;; Make `last-coding-system-used' have the right value.
-	(when (boundp 'last-coding-system-used)
-	  (setq last-coding-system-used coding-system-used))))))
+      (delete-file tmpfil)
+      (unless (equal curbuf (current-buffer))
+	(error "Buffer has changed from `%s' to `%s'"
+	       curbuf (current-buffer)))
+      (when (eq visit t)
+	(set-visited-file-modtime)))))
 
 
 ;; Internal file name functions
@@ -677,14 +661,13 @@ If CONVERT is non-nil exchange \"/\" by \"\\\\\"."
 Either the shares are listed, or the `dir' command is executed.
 Only entries matching the localname are returned.
 Result is a list of (LOCALNAME MODE SIZE MONTH DAY TIME YEAR)."
-  (save-excursion
+  (with-current-buffer (tramp-get-buffer tramp-smb-method user host)
     (save-match-data
       (let ((base (or (and (> (length localname) 0)
 			   (string-match "\\([^/]+\\)$" localname)
 			   (regexp-quote (match-string 1 localname)))
 		      ""))
 	    res entry)
-	(set-buffer (tramp-get-buffer tramp-smb-method user host))
 	(if (and (not share) tramp-smb-share-cache)
 	    ;; Return cached shares
 	    (setq res tramp-smb-share-cache)
@@ -879,21 +862,17 @@ If it doesn't exist, generate a new one."
   "Send the COMMAND to USER at HOST (logged into an SMB session).
 Erases temporary buffer before sending the command.  Returns nil if
 there has been an error message from smbclient."
-  (save-excursion
-    (set-buffer (tramp-get-buffer tramp-smb-method user host))
-    (erase-buffer)
-    (tramp-send-command tramp-smb-method user host command nil t)
-    (tramp-smb-wait-for-output user host)))
+  (tramp-message-for-buffer tramp-smb-method user host 9 "%s" command)
+  (tramp-send-string tramp-smb-method user host command)
+  (tramp-smb-wait-for-output user host))
 
 (defun tramp-smb-maybe-open-connection (user host share)
   "Maybe open a connection to HOST, logging in as USER, using `tramp-smb-program'.
 Does not do anything if a connection is already open, but re-opens the
 connection if a previous connection has died for some reason."
-  (let ((process-connection-type tramp-process-connection-type)
-	(p (get-buffer-process
-	    (tramp-get-buffer tramp-smb-method user host))))
-    (save-excursion
-      (set-buffer (tramp-get-buffer tramp-smb-method user host))
+  (with-current-buffer (tramp-get-buffer tramp-smb-method user host)
+    (let ((process-connection-type tramp-process-connection-type)
+	  (p (get-buffer-process (current-buffer))))
       ;; Check whether it is still the same share
       (unless (and p (processp p) (string-equal tramp-smb-share share))
 	(when (and p (processp p))
@@ -906,11 +885,11 @@ connection if a previous connection has died for some reason."
 		 p (processp p) (memq (process-status p) '(run open)))
 	(unless (and p (processp p) (memq (process-status p) '(run open)))
 	  (delete-process p)
-	  (setq p nil))))
-    (unless (and p (processp p) (memq (process-status p) '(run open)))
-      (when (and p (processp p))
-        (delete-process p))
-      (tramp-smb-open-connection user host share))))
+	  (setq p nil)))
+      (unless (and p (processp p) (memq (process-status p) '(run open)))
+	(when (and p (processp p))
+	  (delete-process p))
+	(tramp-smb-open-connection user host share)))))
 
 (defun tramp-smb-open-connection (user host share)
   "Open a connection using `tramp-smb-program'.
@@ -950,28 +929,27 @@ Domain names in USER and port numbers in HOST are acknowledged."
       (when port   (setq args (append args (list "-p" port))))
 
       ; OK, let's go
-      (tramp-pre-connection tramp-smb-method user host)
+      (tramp-pre-connection tramp-smb-method user host tramp-chunksize)
       (tramp-message
-       7 "Opening connection for //%s@%s/%s..." user host (or share ""))
+       3 "Opening connection for //%s@%s/%s..." user host (or share ""))
 
       (let* ((default-directory (tramp-temporary-file-directory))
 	     (coding-system-for-read nil)
 	     (p (apply #'start-process (buffer-name buffer) buffer
 		       tramp-smb-program args)))
 
-	(tramp-message 9 "Started process %s" (process-command p))
+	(tramp-message 3 "Started process %s" (process-command p))
 	(tramp-set-process-query-on-exit-flag p nil)
 	(set-buffer buffer)
 	(setq tramp-smb-share share)
 
         ; send password
 	(when real-user
-	  (let ((pw-prompt "Password:"))
-	    (tramp-message 9 "Sending password")
-	    (tramp-enter-password p pw-prompt user host)))
+	  (tramp-message 3 "Sending password")
+	  (tramp-enter-password p))
 
 	(unless (tramp-smb-wait-for-output user host)
-	  (tramp-clear-passwd user host)
+	  (tramp-clear-passwd tramp-smb-method user host)
 	  (error "Cannot open connection //%s@%s/%s"
 		 user host (or share "")))))))
 
@@ -979,36 +957,32 @@ Domain names in USER and port numbers in HOST are acknowledged."
 (defun tramp-smb-wait-for-output (user host)
   "Wait for output from smbclient command.
 Returns nil if an error message has appeared."
-  (let ((proc (get-buffer-process (current-buffer)))
-	(found (progn (goto-char (point-min))
-		      (re-search-forward tramp-smb-prompt nil t)))
-	(err   (progn (goto-char (point-min))
-		      (re-search-forward tramp-smb-errors nil t))))
+  (with-current-buffer (tramp-get-buffer tramp-smb-method user host)
+    (let ((proc (get-buffer-process (current-buffer)))
+	  (found (progn (goto-char (point-min))
+			(re-search-forward tramp-smb-prompt nil t)))
+	  (err   (progn (goto-char (point-min))
+			(re-search-forward tramp-smb-errors nil t))))
 
-    ;; Algorithm: get waiting output.  See if last line contains
-    ;; tramp-smb-prompt sentinel or tramp-smb-errors strings.
-    ;; If not, wait a bit and again get waiting output.
-    (while (and (not found) (not err))
+      ;; Algorithm: get waiting output.  See if last line contains
+      ;; tramp-smb-prompt sentinel or tramp-smb-errors strings.
+      ;; If not, wait a bit and again get waiting output.
+      (while (and (not found) (not err))
 
-      ;; Accept pending output.
-      (accept-process-output proc)
+	;; Accept pending output.
+	(tramp-accept-process-output proc)
 
-      ;; Search for prompt.
-      (goto-char (point-min))
-      (setq found (re-search-forward tramp-smb-prompt nil t))
+	;; Search for prompt.
+	(goto-char (point-min))
+	(setq found (re-search-forward tramp-smb-prompt nil t))
 
-      ;; Search for errors.
-      (goto-char (point-min))
-      (setq err (re-search-forward tramp-smb-errors nil t)))
+	;; Search for errors.
+	(goto-char (point-min))
+	(setq err (re-search-forward tramp-smb-errors nil t)))
 
-    ;; Add output to debug buffer if appropriate.
-    (when tramp-debug-buffer
-      (append-to-buffer
-       (tramp-get-debug-buffer tramp-smb-method user host)
-       (point-min) (point-max)))
-
-    ;; Return value is whether no error message has appeared.
-    (not err)))
+      ;; Return value is whether no error message has appeared.
+      (tramp-message 9 "\n%s" (buffer-string))
+      (not err))))
 
 
 ;; Snarfed code from time-date.el and parse-time.el
@@ -1016,11 +990,12 @@ Returns nil if an error message has appeared."
 (defconst tramp-smb-half-a-year '(241 17024)
 "Evaluated by \"(days-to-time 183)\".")
 
-(defconst tramp-smb-parse-time-months '(("jan" . 1) ("feb" . 2) ("mar" . 3)
-			    ("apr" . 4) ("may" . 5) ("jun" . 6)
-			    ("jul" . 7) ("aug" . 8) ("sep" . 9)
-			    ("oct" . 10) ("nov" . 11) ("dec" . 12))
-"Alist mapping month names to integers.")
+(defconst tramp-smb-parse-time-months
+  '(("jan" . 1) ("feb" . 2) ("mar" . 3)
+    ("apr" . 4) ("may" . 5) ("jun" . 6)
+    ("jul" . 7) ("aug" . 8) ("sep" . 9)
+    ("oct" . 10) ("nov" . 11) ("dec" . 12))
+  "Alist mapping month names to integers.")
 
 (defun tramp-smb-time-less-p (t1 t2)
   "Say whether time value T1 is less than time value T2."
