@@ -1,6 +1,7 @@
 ;;; tramp-vc.el --- Version control integration for TRAMP.el
 
-;; Copyright (C) 2000, 2001, 2002, 2003, 2004 by Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2001, 2002, 2003, 2004,
+;;   2005 Free Software Foundation, Inc.
 
 ;; Author: Daniel Pittman <daniel@danann.net>
 ;; Keywords: comm, processes
@@ -19,8 +20,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -41,17 +42,15 @@
 ;; Avoid byte-compiler warnings if the byte-compiler supports this.
 ;; Currently, XEmacs supports this.
 (eval-when-compile
-  (when (fboundp 'byte-compiler-options)
-    (let (unused-vars) ; Pacify Emacs byte-compiler
-      (defalias 'warnings 'identity) ; Pacify Emacs byte-compiler
-      (byte-compiler-options (warnings (- unused-vars))))))
+  (when (featurep 'xemacs)
+    (byte-compiler-options (warnings (- unused-vars)))))
 
 ;; -- vc --
 
 ;; This used to blow away the file-name-handler-alist and reinstall
 ;; TRAMP into it. This was intended to let VC work remotely. It didn't,
 ;; at least not in my XEmacs 21.2 install.
-;; 
+;;
 ;; In any case, tramp-run-real-handler now deals correctly with disabling
 ;; the things that should be, making this a no-op.
 ;;
@@ -95,7 +94,7 @@ See `vc-do-command' for more information."
 	(set (make-local-variable 'vc-parent-buffer-name)
 	     (concat " from " (buffer-name camefrom)))
 	(setq default-directory olddir)
-    
+
 	(erase-buffer)
 
 	(mapcar
@@ -228,7 +227,7 @@ Since TRAMP doesn't do async commands yet, this function doesn't, either."
         (if (or (and (stringp file)     (tramp-tramp-file-p file))
                 (and (buffer-file-name) (tramp-tramp-file-p (buffer-file-name))))
             (setq ad-return-value
-                  (apply 'tramp-vc-do-command-new buffer okstatus command 
+                  (apply 'tramp-vc-do-command-new buffer okstatus command
                          file ;(or file (buffer-file-name))
                          flags))
           ad-do-it)))
@@ -241,7 +240,7 @@ Since TRAMP doesn't do async commands yet, this function doesn't, either."
       (if (or (and (stringp file)     (tramp-tramp-file-p file))
               (and (buffer-file-name) (tramp-tramp-file-p (buffer-file-name))))
           (setq ad-return-value
-                (apply 'tramp-vc-do-command buffer okstatus command 
+                (apply 'tramp-vc-do-command buffer okstatus command
                        (or file (buffer-file-name)) last flags))
         ad-do-it)))))
 ;;-)
@@ -295,7 +294,7 @@ Since TRAMP doesn't do async commands yet, this function doesn't, either."
 	    (tramp-wait-for-output)
 	    (setq exec-status (read (current-buffer)))
 	    (message "Command %s returned status %d." command exec-status)))
-      
+
 	;; Maybe okstatus can be `async' here.  But then, maybe the
 	;; async thing is new in Emacs 21, but this function is only
 	;; used in Emacs 20.
@@ -316,7 +315,7 @@ Since TRAMP doesn't do async commands yet, this function doesn't, either."
     (if (or (and (stringp file)     (tramp-tramp-file-p file))
             (and (buffer-file-name) (tramp-tramp-file-p (buffer-file-name))))
         (setq ad-return-value
-              (apply 'tramp-vc-simple-command okstatus command 
+              (apply 'tramp-vc-simple-command okstatus command
                      (or file (buffer-file-name)) args))
       ad-do-it)))
 
@@ -378,17 +377,17 @@ Since TRAMP doesn't do async commands yet, this function doesn't, either."
 
 
 ;; Do we need to advise the vc-user-login-name function anyway?
-;; This will return the correct login name for the owner of a 
+;; This will return the correct login name for the owner of a
 ;; file. It does not deal with the default remote user name...
 ;;
-;; That is, when vc calls (vc-user-login-name), we return the 
+;; That is, when vc calls (vc-user-login-name), we return the
 ;; local login name, something that may be different to the remote
-;; default. 
+;; default.
 ;;
 ;; The remote VC operations will occur as the user that we logged
 ;; in with however - not always the same as the local user.
 ;;
-;; In the end, I did advise the function. This is because, well, 
+;; In the end, I did advise the function. This is because, well,
 ;; the thing didn't work right otherwise ;)
 ;;
 ;; Daniel Pittman <daniel@danann.net>
@@ -406,7 +405,7 @@ filename we are thinking about..."
   ;; boundness-checking into this function?
   (let* ((file (symbol-value 'file))
 	 (remote-uid
-	  ;; With Emacs 21.4, `file-attributes' has got an optional parameter
+	  ;; With Emacs 22, `file-attributes' has got an optional parameter
 	  ;; ID-FORMAT. Handle this case backwards compatible.
 	  (if (and (functionp 'subr-arity)
 		   (= 2 (cdr (funcall (symbol-function 'subr-arity)
@@ -441,11 +440,11 @@ filename we are thinking about..."
 		     (tramp-handle-vc-user-login-name uid)))) ; get the owner name
         ad-do-it)))                     ; else call the original
 
-  
+
 ;; Determine the name of the user owning a file.
 (defun tramp-file-owner (filename)
   "Return who owns FILE (user name, as a string)."
-  (let ((v (tramp-dissect-file-name 
+  (let ((v (tramp-dissect-file-name
 	    (expand-file-name filename))))
     (if (not (file-exists-p filename))
         nil                             ; file cannot be opened
